@@ -1,4 +1,5 @@
 import math
+from functools import lru_cache
 
 import psutil
 import torch
@@ -9,13 +10,16 @@ from torch import einsum, nn
 from classy_imaginary.modules.diffusion.util import checkpoint as checkpoint_eval
 from classy_imaginary.utils import get_device
 
-try:
-    import xformers  # noqa
-    import xformers.ops  # noqa
+XFORMERS_IS_AVAILABLE = False
 
-    XFORMERS_IS_AVAILBLE = True
+try:
+    if get_device() == "cuda":
+        import xformers  # noqa
+        import xformers.ops  # noqa
+
+        XFORMERS_IS_AVAILABLE = True
 except ImportError:
-    XFORMERS_IS_AVAILBLE = False
+    pass
 
 
 ALLOW_SPLITMEM = True
@@ -151,6 +155,11 @@ def get_mem_free_total(device):
     return mem_free_total
 
 
+@lru_cache(maxsize=1)
+def get_mps_gb_ram():
+    return psutil.virtual_memory().total / (1024**3)
+
+
 class CrossAttention(nn.Module):
     def __init__(self, query_dim, context_dim=None, heads=8, dim_head=64, dropout=0.0):
         super().__init__()
@@ -175,7 +184,7 @@ class CrossAttention(nn.Module):
         #     mask = _global_mask_hack.to(torch.bool)
 
         if get_device() == "cuda" or "mps" in get_device():
-            if not XFORMERS_IS_AVAILBLE and ALLOW_SPLITMEM:
+            if not XFORMERS_IS_AVAILABLE and ALLOW_SPLITMEM:
                 return self.forward_splitmem(x, context=context, mask=mask)
 
         h = self.heads
@@ -235,23 +244,34 @@ class CrossAttention(nn.Module):
         tensor_size = q.shape[0] * q.shape[1] * k.shape[1] * q.element_size()
         modifier = 3 if q.element_size() == 2 else 2.5
         mem_required = tensor_size * modifier
+
         steps = 1
 
-        if mem_required > mem_free_total:
-            steps = 2 ** (math.ceil(math.log(mem_required / mem_free_total, 2)))
-
-        if steps > 64:
-            max_res = math.floor(math.sqrt(math.sqrt(mem_free_total / 2.5)) / 8) * 64
-            raise RuntimeError(
-                f"Not enough memory, use lower resolution (max approx. {max_res}x{max_res}). "
-                f"Need: {mem_required / 64 / gb:0.1f}GB free, Have:{mem_free_total / gb:0.1f}GB free"
-            )
-
-        slice_size = q.shape[1] // steps if (q.shape[1] % steps) == 0 else q.shape[1]
-        if get_device() == "mps":
+        if "mps" in get_device():
             # https://github.com/brycedrennan/imaginAIry/issues/175
             # https://github.com/invoke-ai/InvokeAI/issues/1244
-            slice_size = min(slice_size, 2**30)
+            mps_gb = get_mps_gb_ram()
+            factor = 32 / mps_gb
+
+            slice_size = math.floor(2**30 / (q.shape[0] * q.shape[1] * 16 * factor))
+        else:
+            if mem_required > mem_free_total:
+                steps = 2 ** (math.ceil(math.log(mem_required / mem_free_total, 2)))
+
+            if steps > 64:
+                max_res = (
+                    math.floor(math.sqrt(math.sqrt(mem_free_total / 2.5)) / 8) * 64
+                )
+                raise RuntimeError(
+                    f"Not enough memory, use lower resolution (max approx. {max_res}x{max_res}). "
+                    f"Need: {mem_required / 64 / gb:0.1f}GB free, Have:{mem_free_total / gb:0.1f}GB free"
+                )
+            slice_size = (
+                q.shape[1] // steps if (q.shape[1] % steps) == 0 else q.shape[1]
+            )
+
+        # steps = len(range(0, q.shape[1], slice_size))
+        # print(f"Splitting attention into {steps} steps of {slice_size} slices")
 
         for i in range(0, q.shape[1], slice_size):
             end = i + slice_size
@@ -351,7 +371,7 @@ class BasicTransformerBlock(nn.Module):
         disable_self_attn=False,
     ):
         super().__init__()
-        attn_mode = "softmax-xformers" if XFORMERS_IS_AVAILBLE else "softmax"
+        attn_mode = "softmax-xformers" if XFORMERS_IS_AVAILABLE else "softmax"
         assert attn_mode in self.ATTENTION_MODES
         attn_cls = self.ATTENTION_MODES[attn_mode]
         self.disable_self_attn = disable_self_attn
@@ -400,7 +420,7 @@ class SpatialTransformer(nn.Module):
     and reshape to b, t, d.
     Then apply standard transformer action.
     Finally, reshape to image
-    NEW: use_linear for more efficiency instead of the 1x1 convs
+    NEW: use_linear for more efficiency instead of the 1x1 convs.
     """
 
     def __init__(
